<?php

/*
 * Copyright (C) 2015 Champs-Libres <info@champs-libres.coop>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Chill\ActivityBundle\Export\Export;

use Chill\MainBundle\Export\ExportInterface;
use Doctrine\ORM\QueryBuilder;
use Symfony\Component\Security\Core\Role\Role;
use Doctrine\ORM\Query;
use Chill\ActivityBundle\Security\Authorization\ActivityStatsVoter;
use Doctrine\ORM\EntityManagerInterface;

/**
 * This export allow to compute stats on activity duration.
 * 
 * The desired stat must be given in constructor.
 *
 *
 * @author Julien Fastré <julien.fastre@champs-libres.coop>
 */
class StatActivityDuration implements ExportInterface
{
    /**
     *
     * @var EntityManagerInterface
     */
    protected $entityManager;
    
    const SUM = 'sum';
    
    /**
     * The action for this report.
     *
     * @var string
     */
    protected $action;
    
    /**
     * constructor
     * 
     * @param EntityManagerInterface $em
     * @param string $action the stat to perform
     */
    public function __construct(
            EntityManagerInterface $em,
            $action = 'sum'
            )
    {
        $this->entityManager = $em;
        $this->action = $action;
    }
    
    public function buildForm(\Symfony\Component\Form\FormBuilderInterface $builder)
    {
        
    }

    public function getDescription()
    {
        if ($this->action === self::SUM) {
            return "Sum activities duration by various parameters.";
        }
    }

    public function getTitle()
    {
        if ($this->action === self::SUM) {
            return "Sum activity duration";
        }
        
    }

    public function getType()
    {
        return 'activity';
    }

    public function initiateQuery(array $requiredModifiers,  array $acl, array $data = array())
    {
        $centers = array_map(function($el) { return $el['center']; }, $acl);
        $qb = $this->entityManager->createQueryBuilder();
        
        if ($this->action === self::SUM) {
            $select = "SUM(activity.durationTime) AS export_stat_activity";
        }
        
        $qb->select($select)
                ->from('ChillActivityBundle:Activity', 'activity')
                ->join('activity.person',  'person')
                ->join('person.center', 'center')
                ->where($qb->expr()->in('center', ':centers'))
                ->setParameter(':centers', $centers)
                ;
        
        return $qb;
    }
    
    public function supportsModifiers()
    {
        return array('person', 'activity');
    }
    
    public function requiredRole()
    {
        return new Role(ActivityStatsVoter::STATS);
    }

    public function getAllowedFormattersTypes()
    {
        return array(\Chill\MainBundle\Export\FormatterInterface::TYPE_TABULAR);
    }

    public function getLabels($key, array $values, $data)
    {
        if ($key !== 'export_stat_activity') {
            throw new \LogicException("the key $key is not used by this export");
        }
        
        switch ($this->action) {
            case self::SUM:
                $header = "Sum of activities  duration";
        }
        
        return function($value) use ($header) {
            return $value === '_header' ? 
                    $header
                    :
                    $value
                    ;
        };
    }

    public function getQueryKeys($data)
    {
        return array('export_stat_activity');
    }

    public function getResult($qb, $data)
    {
        return $qb->getQuery()->getResult(Query::HYDRATE_SCALAR);
    }

}
