<?php

/*
 * Chill is a software for social workers
 *
 * Copyright (C) 2014-2015, Champs Libres Cooperative SCRLFS,
 * <http://www.champs-libres.coop>, <info@champs-libres.coop>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Chill\ActivityBundle\Form\Type;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Chill\MainBundle\Templating\TranslatableStringHelper;
use Doctrine\ORM\EntityRepository;
use Chill\ActivityBundle\Entity\ActivityType;

/**
 * Description of TranslatableActivityType
 *
 * @author Champs-Libres Coop
 */
class TranslatableActivityType extends AbstractType
{

    /**
     *
     * @var TranslatableStringHelper
     */
    protected $translatableStringHelper;

    protected $activityTypeRepository;

    public function __construct(
            TranslatableStringHelper $helper,
            EntityRepository $activityTypeRepository
            )
    {
        $this->translatableStringHelper = $helper;
        $this->activityTypeRepository = $activityTypeRepository;
    }

    public function getBlockPrefix()
    {
        return 'translatable_activity_type';
    }

    public function getParent()
    {
        return EntityType::class;
    }

    public function buildForm(\Symfony\Component\Form\FormBuilderInterface $builder, array $options) {
        /* @var $qb \Doctrine\ORM\QueryBuilder */
        $qb = $options['query_builder'];

        if ($options['active_only'] === true) {
            $qb->where($qb->expr()->eq('at.active', ':active'));
            $qb->setParameter('active', true, \Doctrine\DBAL\Types\Type::BOOLEAN);
        }
    }

    public function configureOptions(OptionsResolver $resolver)
    {

        $resolver->setDefaults(
            array(
                'class' => 'ChillActivityBundle:ActivityType',
                'active_only' => true,
                'query_builder' => $this->activityTypeRepository
                    ->createQueryBuilder('at'),
                'choice_label' => function (ActivityType $type) {
                    return $this->translatableStringHelper->localize($type->getName());
                }
            )
        );
    }
}
