<?php

namespace Chill\MainBundle\Serializer\Normalizer;

use Chill\MainBundle\Serializer\Model\Collection;
use Symfony\Component\Serializer\Normalizer\NormalizerInterface;
use Symfony\Component\Serializer\Normalizer\NormalizerAwareInterface;
use Symfony\Component\Serializer\Normalizer\NormalizerAwareTrait;

class CollectionNormalizer implements NormalizerInterface, NormalizerAwareInterface
{
    use NormalizerAwareTrait; 

    public function supportsNormalization($data, string $format = null): bool
    {
        return $data instanceof Collection;
    }

    public function normalize($collection, string $format = null, array $context = [])
    {
        /** @var $collection Collection */
        $paginator = $collection->getPaginator();

        $data['count'] = $paginator->getTotalItems();
        $pagination['first'] = $paginator->getCurrentPageFirstItemNumber();
        $pagination['items_per_page'] = $paginator->getItemsPerPage();
        $pagination['next']  = $paginator->hasNextPage() ? 
            $paginator->getNextPage()->generateUrl() : null;
        $pagination['previous'] = $paginator->hasPreviousPage() ?
            $paginator->getPreviousPage()->generateUrl() : null;
        $pagination['more'] = $paginator->hasNextPage();
        $data['pagination'] = $pagination;

        // normalize results
        $data['results'] = $this->normalizer->normalize($collection->getItems(),
            $format, $context);

        return $data;
    }
}
