<?php

namespace Chill\PersonBundle\Tests\Controller;

use Chill\PersonBundle\Entity\AccompanyingPeriod;
use Chill\MainBundle\Test\PrepareClientTrait;
use Chill\PersonBundle\Entity\Household\Household;
use Symfony\Component\HttpFoundation\Request;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Test\WebTestCase;


class HouseholdApiControllerTest extends WebTestCase
{

    use PrepareClientTrait;

    /**
     * @dataProvider generatePersonId
     */
    public function testSuggestByAccompanyingPeriodParticipation(int $personId)
    {
        $client = $this->getClientAuthenticated();

        $client->request(
            Request::METHOD_GET,
            "/api/1.0/person/household/suggest/by-person/{$personId}/through-accompanying-period-participation.json"
        );

        $this->assertResponseIsSuccessful();
    }

    /**
     * @dataProvider generateHouseholdId
     */
    public function testSuggestAddressByHousehold(int $householdId)
    {
        $client = $this->getClientAuthenticated();

        $client->request(
            Request::METHOD_GET,
            "/api/1.0/person/address/suggest/by-household/{$householdId}.json"
        );

        $this->assertResponseIsSuccessful();
    }

    public function generatePersonId()
    {
        self::bootKernel();

        $qb = self::$container->get(EntityManagerInterface::class)
            ->createQueryBuilder();

        $period = $qb
            ->select('ap')
            ->from(AccompanyingPeriod::class, 'ap')
            ->where(
                $qb->expr()->gte('SIZE(ap.participations)', 2)
            )
            ->getQuery()
            ->setMaxResults(1)
            ->getSingleResult()
            ;

        $person = $period->getParticipations()
            ->first()->getPerson(); 

        yield [ $person->getId() ];
    }

    public function generateHouseholdId()
    {
        self::bootKernel();

        $qb = self::$container->get(EntityManagerInterface::class)
            ->createQueryBuilder();

        $householdIds = $qb
            ->select('DISTINCT household.id')
            ->from(Household::class, 'household')
            ->join('household.members', 'members')
            ->join('members.person', 'person')
            ->join('person.center', 'center')
            ->where($qb->expr()->eq('center.name', ':center_name'))
            ->andWhere($qb->expr()->gt('SIZE(person.accompanyingPeriodParticipations)', 0))
            ->setParameter('center_name', 'Center A')
            ->setMaxResults(100)
            ->getQuery()
            ->getResult()
        ;

        \shuffle($householdIds);

        yield [ \array_pop($householdIds)['id'] ];
        yield [ \array_pop($householdIds)['id'] ];
        yield [ \array_pop($householdIds)['id'] ];
    }
}
